@@ -27,11 +27,12 @@ Define *single* and *useful* ``__version__`` of a project.
 """
 
 
-__version__ = (1, 3, 2, "final", 0)
+__version__ = (1, 4, 0, "final", 0)
 
 
-import os
+import inspect
 import operator
+import os
 
 
 class Version(tuple):
@@ -82,6 +83,22 @@ class Version(tuple):
         """
         return cls(*version_tuple)
 
+    @classmethod
+    def from_tuple_and_hint(cls, version_tuple, hint):
+        """
+        Similar to :meth:`versiontools.Version.from_tuple` but uses the hint
+        object to locate the source tree if needed. A good candidate for hint
+        object is the module that contains the version_tuple.
+
+        .. versionadded:: 1.4
+        """
+        self = cls.from_tuple(version_tuple)
+        if self._source_tree is None:
+            path = inspect.getsourcefile(hint)
+            if path is not None:
+                self._source_tree = os.path.dirname(os.path.abspath(path))
+        return self
+
     def __new__(cls, major, minor, micro=0, releaselevel="final", serial=0):
         if releaselevel not in ('dev', 'alpha', 'beta', 'candidate', 'final'):
             raise ValueError(
@@ -128,7 +145,6 @@ class Version(tuple):
         Find the absolute pathname of the tree that contained the file
         that called our __init__()
         """
-        import inspect
         frame = inspect.currentframe()
         outer_frames = inspect.getouterframes(frame)
         for index0, record in enumerate(outer_frames):
@@ -189,7 +205,7 @@ class Version(tuple):
         return version
 
 
-def format_version(version):
+def format_version(version, hint=None):
     """
     Pretty formatting for 5-element version tuple.
 
@@ -197,6 +213,8 @@ def format_version(version):
     """
     if isinstance(version, Version):
         return str(version)
+    elif isinstance(version, tuple) and len(version) == 5 and hint is not None:
+        return str(Version.from_tuple_and_hint(version, hint))
     elif isinstance(version, tuple) and len(version) == 5:
         return str(Version.from_tuple(version))
     else:
@@ -227,8 +245,12 @@ def handle_version(dist, attr, value):
     if ":" not in value:
         raise DistutilsSetupError(
             "version must be of the form `module_or_package:identifier`")
-    # Import the module or package indicated by the version tag
+    # Parse the version string
     module_or_package, identifier = value.split(":", 1)
+    # Use __version__ unless specified otherwise
+    if identifier == "":
+        identifier = "__version__"
+    # Import the module or package indicated by the version tag
     try:
         obj = __import__(module_or_package, fromlist=[''])
     except ImportError as ex:
@@ -240,4 +262,4 @@ def handle_version(dist, attr, value):
     except AttributeError as ex:
         raise DistutilsSetupError("Unable to access %r in %r: %s" % (identifier, module_or_package, ex))
     # Yay we have it! Let's format it correctly and overwrite the old value
-    dist.metadata.version = format_version(version)
+    dist.metadata.version = format_version(version, obj)
